import toml

from ..api import API
from ..cases.compliance import (
    TestComplianceValidDummy,
    TestComplianceInValidDummy,
)


class TOMLAPI(API):
    package = toml
    repo = "https://github.com/uiri/toml"


class TOMLTestComplianceValidDummy(TestComplianceValidDummy):
    OPEN_FLAG = "r"


class TOMLTestComplianceInValidDummy(TestComplianceInValidDummy):
    OPEN_FLAG = "r"



import qtoml
from ..api import API
from ..cases.compliance import TestComplianceValidDummy


class QTOMLAPI(API):
    package = qtoml
    repo = "https://github.com/alethiophile/qtoml"


class QTOMLTestComplianceValidDummy(TestComplianceValidDummy):
    OPEN_FLAG = "r"



from typing import Any
from ..case import TestCase, TestCaseDummy


class TestDumpNoneDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)
        try:
            return self.api.dumps(None)
        except Exception as e:
            return e

    def result(self, out: Any) -> str:
        replace_newline = lambda s: s.replace("\n", " ")
        if isinstance(out, Exception):
            return f"Raises {replace_newline(str(out))}"
        return f"Dumps to `{out!r}`"


class TestDumpNone(TestCase):
    """How the package dumps `None` value in python,
    literally `<package>.dumps(None)`
    """

    HEADER = "Dumped value or error"
    DUMMY_CLASS = TestDumpNoneDummy


class TestDumpListWithNoneDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)
        try:
            return self.api.dumps({"key": [1, 2, 3, None, 5]})
        except Exception as e:
            return e

    def result(self, out: Any) -> str:
        replace_newline = lambda s: s.replace("\n", " ")
        if isinstance(out, Exception):
            return f"Raises {replace_newline(str(out))}"
        return f"Dumps to `{out!r}`"


class TestDumpListWithNone(TestCase):
    """How the package dumps a list with `None` value in it.
    Literally `<package>.dumps({"key": [1, 2, 3, None, 5]})`
    """
    HEADER = "Dumped value or error"
    DUMMY_CLASS = TestDumpListWithNoneDummy


class TestDumpValueNoneDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)
        try:
            return self.api.dumps({"key": None})
        except Exception as e:
            return e

    def result(self, out: Any) -> str:
        replace_newline = lambda s: s.replace("\n", " ")
        if isinstance(out, Exception):
            return f"Raises {replace_newline(str(out))}"
        if out == "":
            return "Ignores the key (dumps to an empty string)"
        return f"Dumps to `{replace_newline(repr(out))}`"


class TestDumpValueNone(TestCase):
    """How the package dumps key-value pair with value `None`,
    literally `<package>.dumps({"key": None})`
    """

    HEADER = "Dumped value or error"
    DUMMY_CLASS = TestDumpValueNoneDummy


class TestLoadNoneLikeDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)
        try:
            return self.api.loads('v1 = "null"\nv2 = "None"')
        except Exception as e:
            return e

    def result(self, out: Any) -> str:
        return f"`{out!r}`"


class TestLoadNoneLike(TestCase):
    """How the package loads `None`-like value in string,
    literally `<package>.loads('v1 = "null"\nv2 = "None"')`
    """

    HEADER = "Loaded as"
    DUMMY_CLASS = TestLoadNoneLikeDummy



from pyparam import Params

from .suite import TestSuite
from .utils import logger


def _init_params() -> Params:
    params = Params(help_on_void=False)
    params.add_param(
        "datadir",
        type="path",
        default="/tmp/toml-bench",
        desc="Where to put the test-data",
    )
    params.add_param(
        "report",
        type="file",
        desc=(
            "If provided, the report will be written to "
            "this file under section `## Report` in markdown format. "
            "If the section exists, it will be overwritten."
        ),
    )
    params.add_param(
        "comver",
        default="1.1.0",
        desc="The version of the toml-test to use in compliance tests",
    )
    params.add_param(
        "k,keyword",
        default="",
        desc=(
            "The keyword to limit the tests with name or package name "
            "including the keyword (case-insensitive)"
        ),
    )
    params.add_param(
        "iter",
        default=5000,
        desc="The number of iterations to run in speed tests",
    )
    return params


def main():
    parsed = _init_params().parse()
    logger.info("Data directory: %s", parsed.datadir)
    suite = TestSuite()
    suite.set_args(parsed)
    suite.prepare()
    suite.run()
    suite.result()



import rtoml
from ..api import API
from ..cases.compliance import (
    TestComplianceValidDummy,
    TestComplianceInValidDummy,
)


class TOMLKitAPI(API):
    package = rtoml
    repo = "https://github.com/samuelcolvin/rtoml"


class RTOMLTestComplianceValidDummy(TestComplianceValidDummy):
    OPEN_FLAG = "r"


class RTOMLTestComplianceInValidDummy(TestComplianceInValidDummy):
    OPEN_FLAG = "r"



from regex import P
import tomlkit
from ..api import API


class TOMLKitAPI(API):
    package = tomlkit
    repo = "https://github.com/sdispater/tomlkit"

    def load(self, f):
        content = f.read()
        return tomlkit.loads(content)



from typing import Any
import importlib_metadata as im
import tomli
import tomli_w
from ..api import API
from ..cases.unicode import TestLoadUnicodeDummy


class TOMLiAPI(API):
    package = tomli_w
    repo = "https://github.com/hukkin/tomli"
    loads = lambda self, data: tomli.loads(data)
    load = lambda self, f: tomli.load(f)

    def version(self) -> str:
        return (
            f"{im.version('tomli')}; "
            f"**tomli_w**: {im.version(self.package.__name__)}"
        )


class TOMLiTestLoadUnicodeDummy(TestLoadUnicodeDummy):
    def result(self, out: Any) -> str:
        if isinstance(out, Exception):
            datafile = self.args.datadir / "unicode" / "unicode.toml"
            with open(datafile, "rb") as f:
                try:
                    loaded = self.api.load(f)
                except Exception as e:
                    loaded = e

            return (
                f"```{out.__class__.__name__}: {out} ```<br />"
                "**When load with:**<br />"
                "`with open(datafile, 'rb') as f:`<br />"
                "`　   loaded = self.api.load(f)`<br />"
                "**Yields:**<br />"
                f"`{loaded!r}`"
            )
        return super().result(out)



from typing import Any
from ..case import TestCase, TestCaseDummy
from ..api import APIs


class VersionDummy(TestCaseDummy):
    def run(self, case: "TestCase") -> Any:
        super().run(case)
        return self.api.version()


class Version(TestCase):
    """The verions of the packages tested in this report."""

    ORDER = -99
    HEADER = "Version"
    DUMMY_CLASS = VersionDummy



import sys
import math
import logging
import json
from os import PathLike
from importlib import import_module
from pathlib import Path
from types import ModuleType
from typing import Any, Callable, Mapping, Type

from dateutil import parser

# logger
logger = logging.getLogger("toml-bench")
logger.setLevel(logging.INFO)
stream_handler = logging.StreamHandler(sys.stderr)
stream_handler.setFormatter(
    logging.Formatter(
        "[%(asctime)s][%(levelname)7s] %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )
)
logger.addHandler(stream_handler)


def load_subpkgs(dire: PathLike) -> Mapping[str, ModuleType]:
    """Get all packages and the module from .packages"""
    out = {}
    for path in Path(__file__).parent.joinpath(dire).glob("*.py"):
        if path.name.startswith("_"):
            continue

        out[path.stem] = import_module(
            f".{dire}.{path.stem}",
            package=__package__,
        )

    return out


def get_object(
    module: ModuleType,
    checker: Callable[[Any], bool],
    multi: bool = False,
) -> Any:
    """Get object from a module with a checker"""
    out = [] if multi else None
    for attr in dir(module):
        if attr.startswith("_"):
            continue

        obj = getattr(module, attr)
        if checker(obj):
            if not multi:
                return obj
            out.append(obj)

    return out


def doc_formatter(**kwargs: Any) -> Callable[[Type], Type]:
    """Format docstring of a class"""

    def decorator(cls: Type) -> Type:
        cls.__doc__ = cls.__doc__ % kwargs
        return cls

    return decorator


class NAN:
    def __eq__(self, other):
        return math.isnan(other)

    def __ne__(self, other):
        return not math.isnan(other)


def _cast_value(value: Any) -> Any:
    if isinstance(value, list):
        return [_cast_value(v) for v in value]
    if not isinstance(value, dict):
        return value

    out = value.copy()
    if len(value) == 2 and "value" in value and "type" in value:
        if value["type"] == "integer":
            return int(value["value"])
        if value["type"] == "array":
            return [_cast_value(v) for v in value["value"]]
        if value["type"] == "string":
            return str(value["value"])
        if value["type"] == "float":
            out = float(value["value"])
            if math.isnan(out):
                out = NAN()
            return out
        if value["type"] == "bool":
            return value["value"] == "true"
        if value["type"] == "datetime":
            return parser.isoparse(value["value"])
        if value["type"] == "datetime-local":
            return parser.parse(value["value"])
        if value["type"] == "date":
            return parser.isoparse(value["value"]).date()
        if value["type"] == "date-local":
            return parser.parse(value["value"]).date()
        if value["type"] == "time":
            return parser.isoparse(value["value"]).time()
        if value["type"] == "time-local":
            return parser.parse(value["value"]).time()

        return out["value"]

    for key, val in value.items():
        out[key] = _cast_value(val)
    return out


def load_json_data(file) -> Mapping[str, Any]:
    data = json.load(file)
    return _cast_value(data)



from typing import TYPE_CHECKING

from .utils import load_subpkgs, logger, get_object
from .case import TestCase

if TYPE_CHECKING:
    from pyparam import Namespace


class TestSuite:
    def __init__(self) -> None:
        """Collect cases"""
        self.cases = []
        self.args = None
        for name, case in load_subpkgs("cases").items():
            logger.info("[%s] Collecting test cases ...", name)
            case_classes = get_object(
                case,
                lambda obj: (
                    isinstance(obj, type)
                    and issubclass(obj, TestCase)
                    and obj is not TestCase
                ),
                multi=True,
            )
            for case_class in case_classes:
                case = case_class()
                case.load_dummies()
                self.cases.append(case)

        # put it in the right order
        self.cases.sort(key=lambda x: x.ORDER)

    def set_args(self, args: "Namespace") -> None:
        """Set args"""
        self.args = args
        for case in self.cases:
            case.set_args(args)

        kw = self.args.keyword
        if kw:
            self.cases = [
                case
                for case in self.cases
                if kw.lower() in case.__class__.__name__.lower()
            ]

    def _prepare_datadir(self) -> None:
        """Prepare packages"""
        logger.info(
            "[%s] Preparing data directory ...",
            self.__class__.__name__,
        )
        self.args.datadir.mkdir(parents=True, exist_ok=True)

    def prepare(self) -> None:
        """Prepare all cases"""
        self._prepare_datadir()
        for case in self.cases:
            case.prepare()

    def run(self) -> None:
        """Run all cases"""
        for case in self.cases:
            case.run_all()

    def result(self) -> None:
        """Result of all cases"""
        report = []
        for case in self.cases:
            report.append("")
            report.append(f"### {case.__class__.__name__}")
            report.append("")
            report.append(case.__doc__ or "")
            report.append("")
            report.append(f"|Package|{case.HEADER}|")
            report.append("|:------|:------|")
            report.append(case.result_all())
        report.append("")
        report.append("")

        if self.args.report is None:
            print("\n".join(report))
            return

        logger.info(
            "[%s] Writing report to [%s] ...",
            self.__class__.__name__,
            self.args.report,
        )
        out = []
        report_hit = False
        if self.args.report.exists():
            with open(self.args.report, "r") as f:
                for line in f:
                    line = line.rstrip()
                    if line == "## Report":
                        out.append(line)
                        out.extend(report)
                        report_hit = True
                    elif (
                        # Next session or links
                        (line.startswith("## ") or line.startswith("["))
                        and report_hit
                    ):
                        out.append(line)
                        report_hit = False
                    elif not report_hit:
                        out.append(line)
        else:
            out.append("## Report")
            out.extend(report)

        out.append("")

        with open(self.args.report, "w") as f:
            f.write("\n".join(out))



from pathlib import Path
from importlib import import_module

for path in Path(__file__).parent.glob("*.py"):
    if path.name.startswith("_"):
        continue

    import_module(f".{path.stem}", package=__package__)



from abc import ABC
from types import ModuleType
import importlib_metadata as im

from typing import Any, Mapping, TYPE_CHECKING


if TYPE_CHECKING:
    from os import PathLike


APIs = {}


class API(ABC):

    package = None
    repo = ""

    @classmethod
    def __init_subclass__(cls) -> None:
        APIs[cls.__module__.split(".")[-1]] = cls(cls.package)

    def __init__(self, package: ModuleType) -> None:
        self.package = package

    def load(self, path: "PathLike") -> Mapping[str, Any]:
        return self.package.load(path)

    def dumps(self, data: Mapping[str, Any]) -> str:
        return self.package.dumps(data)

    def loads(self, data: str) -> Mapping[str, Any]:
        return self.package.loads(data)

    def dump(self, data: Mapping[str, Any], path: "PathLike") -> None:
        self.package.dump(data, path)

    def version(self) -> str:
        return im.version(self.package.__name__)



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any

from .utils import get_object, load_subpkgs, logger
from .api import APIs, API

if TYPE_CHECKING:
    from pyparam import Namespace


class TestCaseDummy(ABC):
    def __init__(self) -> None:
        self.args = None
        self.pkgname = None

    @property
    def api(self) -> API:
        return APIs[self.pkgname]

    @abstractmethod
    def run(self, case: "TestCase") -> Any:
        logger.info(
            "[%s][%s] Running test case ...",
            case.__class__.__name__,
            self.pkgname,
        )

    def result(self, out: Any) -> str:
        return str(out)


class TestCase(ABC):

    ORDER = 0
    HEADER = "Result"
    DUMMY_CLASS = TestCaseDummy

    def __init__(self) -> None:
        self.results = {}
        self.dummies = []
        self.args = None

    def set_args(self, args: "Namespace") -> None:
        self.args = args
        for dummy in self.dummies:
            dummy.args = args

    def load_dummies(self):
        for name, pkg in load_subpkgs("packages").items():
            dummy_class = get_object(
                pkg,
                lambda obj: (
                    isinstance(obj, type)
                    and issubclass(obj, self.__class__.DUMMY_CLASS)
                    and obj is not self.__class__.DUMMY_CLASS
                ),
            )
            if dummy_class is None:
                dummy_class = self.__class__.DUMMY_CLASS
            dummy = dummy_class()
            dummy.pkgname = name
            self.dummies.append(dummy)

    def prepare(self) -> None:
        logger.info("[%s] Preparing test case ...", self.__class__.__name__)

    def run_all(self) -> None:
        for dummy in self.dummies:
            result = dummy.run(self)
            self.results[dummy.pkgname] = dummy.result(result)

    def result_all(self) -> str:
        ret = []
        for dummy in self.dummies:
            out = self.results[dummy.pkgname]
            ret.append(f"|[{dummy.pkgname}]({dummy.api.repo})|{out}|")
        return "\n".join(ret)



from typing import Any
from ..case import TestCase, TestCaseDummy


class TestDumpsUnicodeDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)
        v1 = {"你好": "世界"}
        try:
            return self.api.dumps(v1)
        except Exception as e:
            return e

    def result(self, out: Any) -> str:
        replace_newline = lambda s: s.replace("\n", " ")
        return f"`{replace_newline(str(out))}`"


class TestDumpsUnicode(TestCase):
    """How the package dumps Unicode in python,
    literally, `<api>.dumps({"你好": "世界"})`"""

    HEADER = "Dumped value"
    DUMMY_CLASS = TestDumpsUnicodeDummy


class TestLoadUnicodeDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)
        try:
            with open(case.datafile, "r", encoding="utf-8") as f:
                return self.api.load(f)
        except Exception as e:
            return e

    def result(self, out: Any) -> str:
        replace_newline = lambda s: s.replace("\n", " ")
        return f"`{replace_newline(str(out))}`"


class TestLoadUnicode(TestCase):
    """How the package loads a file with unicode.

    # Create a file with unicode content
    with open(self.datafile, "w", encoding="utf-8") as f:
        f.write('"你好" = "世界"\\n')

    # Use `<api>.load()` to load the file
    with open(self.datafile, "r", encoding="utf-8") as f:
        loaded = self.api.load(f)
    """

    HEADER = "Loaded as"
    DUMMY_CLASS = TestLoadUnicodeDummy

    def __init__(self) -> None:
        super().__init__()
        self.datafile = None

    def prepare(self) -> None:
        super().prepare()
        self.datafile = self.args.datadir / "unicode" / "unicode.toml"
        self.datafile.parent.mkdir(parents=True, exist_ok=True)
        with open(self.datafile, "w", encoding="utf-8") as f:
            f.write('"你好" = "世界"\n')



import timeit
import urllib.request
from typing import Any
from ..case import TestCase, TestCaseDummy
from ..utils import doc_formatter

PYTOMLPP_DATA_URL = (
    "https://github.com/bobfang1992/pytomlpp/raw/master/benchmark/data.toml"
)

TOMLI_DATA_URL = (
    "https://github.com/hukkin/tomli/raw/master/benchmark/data.toml"
)

RTOML_DATA_URL = (
    "https://github.com/samuelcolvin/rtoml/raw/main/benchmarks/data.toml"
)


class TestSpeedDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)

        data = case.datafile.read_text()
        return timeit.timeit(
            lambda: self.api.loads(data),
            number=self.args.iter,
        )

    def result(self, out: Any) -> str:
        return f"{out:.2f}s ({self.args.iter} iterations)"


class TestSpeedWithPytomlppDataDummy(TestSpeedDummy):
    ...


@doc_formatter(url=PYTOMLPP_DATA_URL)
class TestSpeedWithPytomlppData(TestCase):
    """Test the speed of loading data provided by pytomlpp.


    %(url)s"""

    DUMMY_CLASS = TestSpeedWithPytomlppDataDummy
    ORDER = 9

    def _prepare_datafile(self, filename: str, url: str) -> None:
        self.datafile = self.args.datadir / "speed" / filename
        self.number = self.args.iter
        if not self.datafile.exists():
            self.datafile.parent.mkdir(parents=True, exist_ok=True)
            with urllib.request.urlopen(url) as resp, self.datafile.open(
                "wb"
            ) as f:
                f.write(resp.read())

    def prepare(self) -> None:
        super().prepare()
        self._prepare_datafile("pytomlpp.toml", PYTOMLPP_DATA_URL)


class TestSpeedWithTomliDataDummy(TestSpeedDummy):
    ...


@doc_formatter(url=TOMLI_DATA_URL)
class TestSpeedWithTomliData(TestSpeedWithPytomlppData):
    """Test the speed of loading data provided by tomli.

    %(url)s"""

    DUMMY_CLASS = TestSpeedWithTomliDataDummy

    def prepare(self) -> None:
        super().prepare()
        self._prepare_datafile("tomli.toml", TOMLI_DATA_URL)


class TestSpeedWithRtomlDataDummy(TestSpeedDummy):
    ...


@doc_formatter(url=RTOML_DATA_URL)
class TestSpeedWithRtomlData(TestSpeedWithPytomlppData):
    """Test the speed of loading data provided by rtoml.

    %(url)s"""

    DUMMY_CLASS = TestSpeedWithRtomlDataDummy

    def prepare(self) -> None:
        super().prepare()
        self._prepare_datafile("rtoml.toml", RTOML_DATA_URL)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['toml_bench', 'toml_bench.cases', 'toml_bench.packages']

package_data = \
{'': ['*']}

install_requires = \
['importlib-metadata>=4.11.3,<5.0.0',
 'pyparam>=0.4.17,<0.5.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'pytomlpp>=1.0.11,<2.0.0',
 'qtoml>=0.3.1,<0.4.0',
 'regex',
 'rtoml>=0.7.1,<0.8.0',
 'toml>=0.10.2,<0.11.0',
 'tomli-w>=1.0.0,<2.0.0',
 'tomli==2.0.1',
 'tomlkit>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['toml-bench = toml_bench.__main__:main']}

setup_kwargs = {
    'name': 'toml-bench',
    'version': '0.0.2',
    'description': 'Benchmarking for python toml libraries',
    'long_description': None,
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import zipfile
import urllib.request
from pathlib import Path
from typing import Any

from ..case import TestCase, TestCaseDummy
from ..utils import doc_formatter, load_json_data


TOML_TEST_REPO = (
    # "https://github.com/BurntSushi/toml-test/archive/refs/tags/v1.1.0.zip"
    "https://github.com/BurntSushi/toml-test/"
)

TEST_FILE_PREFIX = (
    "https://github.com/BurntSushi/toml-test/blob/v%(ver)s/tests/"
)


class TestComplianceDummy(TestCaseDummy):

    OPEN_FLAG = "rb"
    KIND = "valid"

    def __init__(self) -> None:
        super().__init__()
        self.total = 0

    def _runfile(
        self,
        tomlfile: Path,
        subdir: bool = False,
    ) -> Any:
        self.total += 1
        filename = (
            tomlfile.name
            if not subdir
            else f"{tomlfile.parent.name}/{tomlfile.name}"
        )
        url = (
            f"{TEST_FILE_PREFIX % {'ver': self.args.comver}}"
            f"/{self.__class__.KIND}/{filename}"
        )
        with tomlfile.open(self.__class__.OPEN_FLAG) as f:
            try:
                data = self.api.load(f)
            except Exception as e:
                return Exception(f"[{filename}]({url}) {e}")
        with tomlfile.with_suffix(".json").open("r") as f:
            expect = load_json_data(f)

        try:
            assert (
                data == expect
            ), f"[{filename}]({url}) Parsed as unexpected data."
        except AssertionError as e:
            return e

    def run(self, case: "TestCase") -> Any:
        super().run(case)
        errors = []
        kind = self.__class__.KIND
        for tomlfile in case.datadir.joinpath("tests", kind).glob("*.toml"):
            out = self._runfile(tomlfile)
            if out is not None:
                errors.append(out)

        for tomlfile in case.datadir.joinpath("tests", kind).glob("*/*.toml"):
            out = self._runfile(tomlfile, subdir=True)
            if out is not None:
                errors.append(out)
        return errors

    def result(self, out: Any) -> str:
        if not out:
            return f"OK, *{self.total}/{self.total} (100%) passed*"

        passed = self.total - len(out)
        out.append(
            f"*{passed}/{self.total} "
            f"({100.0 * passed / self.total:.2f}%) passed*"
        )

        return "<br />".join(str(e) for e in out)


class TestComplianceValidDummy(TestComplianceDummy):
    KIND = "valid"


@doc_formatter(url=TOML_TEST_REPO)
class TestComplianceValid(TestCase):
    """Test the compliance with the standard test suites for
        valid toml files here:

    %(url)s

The tests come up with a JSON counterpart that can be used to valid whether
loading the toml file yields the same result as the JSON counterpart.
    """

    ORDER = 1
    DUMMY_CLASS = TestComplianceValidDummy

    def _prepare_data(self) -> None:
        ver = self.args.comver
        self.HEADER = f"{self.HEADER} (BurntSushi/toml-test v{ver})"
        datafile = self.args.datadir / "compliance" / f"toml-test-{ver}.zip"
        self.datadir = self.args.datadir / "compliance" / f"toml-test-{ver}"
        self.datadir.parent.mkdir(parents=True, exist_ok=True)
        url = f"{TOML_TEST_REPO}/archive/refs/tags/v{ver}.zip"
        if not self.datadir.exists():
            if not datafile.exists():
                with urllib.request.urlopen(url) as resp, datafile.open(
                    "wb"
                ) as f:
                    f.write(resp.read())
            with zipfile.ZipFile(datafile) as zf:
                zf.extractall(self.datadir.parent)

    def prepare(self) -> None:
        super().prepare()
        self._prepare_data()


class TestComplianceInValidDummy(TestComplianceDummy):

    KIND = "invalid"

    def _runfile(self, tomlfile: Path, subdir: bool = False) -> Any:
        self.total += 1
        filename = (
            tomlfile.name
            if not subdir
            else f"{tomlfile.parent.name}/{tomlfile.name}"
        )
        url = (
            f"{TEST_FILE_PREFIX % {'ver': self.args.comver}}/invalid/{filename}"
        )
        with tomlfile.open(self.__class__.OPEN_FLAG) as f:
            try:
                self.api.load(f)
            except Exception as e:
                return Exception(f"[{filename}]({url}) {e}")
            else:
                return f"Not OK: [{filename}]({url}) incorrectly parsed."

    def result(self, out: Any) -> str:
        replace_newline = lambda s: s.replace("\n", " ")
        passed = [isinstance(e, Exception) for e in out]
        if all(passed):
            return (
                f"OK: {replace_newline(str(out[-1]))}<br /> "
                f"*{len(passed)}/{len(passed)} (100%) passed*"
            )
        out.append(
            f"*{sum(passed)}/{self.total} "
            f"({100.0 * sum(passed) / self.total:.2f}%) passed*"
        )
        return "<br />".join(
            replace_newline(e) for e in out if isinstance(e, str)
        )


@doc_formatter(url=TOML_TEST_REPO)
class TestComplianceInvalid(TestComplianceValid):
    """Test the compliance with the standard test suites for
        invalid toml files here:

    %(url)s

- `Not OK`: The toml file is parsed without error, but expected to fail.
- `OK`: All files are failed to parse, as expected. Showing the last parsing
    error.
    """

    ORDER = 2
    DUMMY_CLASS = TestComplianceInValidDummy



from typing import Any
from ..case import TestCase, TestCaseDummy


class TestDumpKeysOrderDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)

        v1 = {"a": 1, "b": 2, "c": 3}
        v2 = {"b": 1, "c": 2, "a": 3}
        v3 = {"c": 1, "a": 2, "b": 3}

        return [self.api.dumps(v1), self.api.dumps(v2), self.api.dumps(v3)]

    def result(self, out: Any) -> str:
        e1 = "a = 1\nb = 2\nc = 3\n"
        e2 = "b = 1\nc = 2\na = 3\n"
        e3 = "c = 1\na = 2\nb = 3\n"
        if out[0] == e1 and out[1] == e2 and out[2] == e3:
            return "Kept"
        return "Lost"


class TestDumpKeysOrder(TestCase):
    """Whether the package preserves the order of the keys while dumps
    a python dictionary. Thus, whether
    `<package>.dumps({"c": 1, "a": 2, "b": 3})` yields
    a string like `c = 1\\na = 2\\nb = 3\\n`.
    """

    HEADER = "Keys order kept?"
    DUMMY_CLASS = TestDumpKeysOrderDummy


class TestLoadKeysOrderDummy(TestCaseDummy):
    def run(self, case: TestCase) -> Any:
        super().run(case)

        v1 = "a = 1\nb = 2\nc = 3\n"
        v2 = "b = 1\nc = 2\na = 3\n"
        v3 = "c = 1\na = 2\nb = 3\n"

        return [self.api.loads(v1), self.api.loads(v2), self.api.loads(v3)]

    def result(self, out: Any) -> str:
        e1 = ["a", "b", "c"]
        e2 = ["b", "c", "a"]
        e3 = ["c", "a", "b"]

        if list(out[0]) == e1 and list(out[1]) == e2 and list(out[2]) == e3:
            return "Kept"
        return "Lost"


class TestLoadKeysOrder(TestCase):
    """Whether the package preserves the order of the keys while loads
    a TOML string. Thus, whether
    `<package>.loads('c = 1\na = 2\nb = 3\n')` yields a dictionary
    with keys in the order of `['c', 'a', 'b']`.
    """

    HEADER = "Keys order kept?"
    DUMMY_CLASS = TestLoadKeysOrderDummy
